@@ -753,201 +753,87 @@ NP_Initialize(NPNetscapeFuncs* nsTable, NPPluginFuncs* pluginFuncs)
 {
     NPError err = NPERR_NO_ERROR;
 
+    /* Zero everything */
+    memset(&gNetscapeFuncs, 0, sizeof(gNetscapeFuncs));
+
     /* validate input parameters */
-
-    if ((nsTable == NULL) || (pluginFuncs == NULL))
+    if(nsTable != NULL)
     {
-        err = NPERR_INVALID_FUNCTABLE_ERROR;
-    }
+        uint32_t size;
 
-    /*
-     * Check the major version passed in Netscape's function table.
-     * We won't load if the major version is newer than what we expect.
-     * Also check that the function tables passed in are big enough for
-     * all the functions we need (they could be bigger, if Netscape added
-     * new APIs, but that's OK with us -- we'll just ignore them).
-     *
-     */
+        /*
+        * Check the major version passed in Netscape's function table.
+        */
 
-    if (err == NPERR_NO_ERROR) 
-    {
         if ((nsTable->version >> 8) > NP_VERSION_MAJOR)
         {
             err = NPERR_INCOMPATIBLE_VERSION_ERROR;
         }
 
-        if (nsTable->size < ((char *)&nsTable->getvalue - (char *)nsTable))
+        if (nsTable->size > sizeof(NPNetscapeFuncs))
         {
-            err = NPERR_INVALID_FUNCTABLE_ERROR;
+            /* Looks like more functions provided than we know about..*/
+            size = sizeof(NPNetscapeFuncs);
+        }
+        else
+        {
+            /* Copy across only those entries that were passed in (i.e. size) */
+            size = nsTable->size;
         }
 
-        if (pluginFuncs->size < ((char*)&pluginFuncs->getvalue - (char*)pluginFuncs))
-        {
-            err = NPERR_INVALID_FUNCTABLE_ERROR;
-        }
+        memcpy(&gNetscapeFuncs, nsTable, size);
+        gNetscapeFuncs.size = size;
     }
-        
-    
-    if (err == NPERR_NO_ERROR) 
+    else
     {
-        /* Zero everything */
-        memset(&gNetscapeFuncs, 0, sizeof(gNetscapeFuncs));
-        /*
-         * Copy all the fields of Netscape function table into our
-         * copy so we can call back into Netscape later.  Note that
-         * we need to copy the fields one by one, rather than assigning
-         * the whole structure, because the Netscape function table
-         * could actually be bigger than what we expect.
-         */
-        gNetscapeFuncs.version       = nsTable->version;
-        gNetscapeFuncs.size          = nsTable->size;
-        gNetscapeFuncs.posturl       = nsTable->posturl;
-        gNetscapeFuncs.geturl        = nsTable->geturl;
-        gNetscapeFuncs.geturlnotify  = nsTable->geturlnotify;
-        gNetscapeFuncs.requestread   = nsTable->requestread;
-        gNetscapeFuncs.newstream     = nsTable->newstream;
-        gNetscapeFuncs.write         = nsTable->write;
-        gNetscapeFuncs.destroystream = nsTable->destroystream;
-        gNetscapeFuncs.status        = nsTable->status;
-        gNetscapeFuncs.uagent        = nsTable->uagent;
-        gNetscapeFuncs.memalloc      = nsTable->memalloc;
-        gNetscapeFuncs.memfree       = nsTable->memfree;
-        gNetscapeFuncs.memflush      = nsTable->memflush;
-        gNetscapeFuncs.reloadplugins = nsTable->reloadplugins;
+        err = NPERR_INVALID_FUNCTABLE_ERROR;
+    }
+    
+    if (pluginFuncs != NULL) 
+    {
+        /* Create a temporary local copy */
+        NPPluginFuncs funcs;
 
-        gNetscapeFuncs.posturlnotify = nsTable->posturlnotify;
-        gNetscapeFuncs.getvalue      = nsTable->getvalue;
-
-        /* Version 0.9?? */
-        if (nsTable->size >= ((char *)&nsTable->setvalue - (char *)nsTable))
-        {
-            gNetscapeFuncs.setvalue  = nsTable->setvalue;
-        }
-
-        /* Version 0.11 */
-        if (nsTable->size >= ((char *)&nsTable->forceredraw - (char *)nsTable))
-        {
-            gNetscapeFuncs.invalidaterect = nsTable->invalidaterect;
-            gNetscapeFuncs.invalidateregion = nsTable->invalidateregion;
-            gNetscapeFuncs.forceredraw = nsTable->forceredraw;
-        }
-
-        /* Version 0.14 */
-        if (nsTable->size >= ((char *)&nsTable->setexception - (char *)nsTable))
-        {
-          gNetscapeFuncs.getstringidentifier = nsTable->getstringidentifier;
-          gNetscapeFuncs.getstringidentifiers = nsTable->getstringidentifiers;
-          gNetscapeFuncs.getintidentifier = nsTable->getintidentifier;
-          gNetscapeFuncs.identifierisstring = nsTable->identifierisstring;
-          gNetscapeFuncs.utf8fromidentifier = nsTable->utf8fromidentifier;
-          gNetscapeFuncs.intfromidentifier = nsTable->intfromidentifier;
-          gNetscapeFuncs.createobject = nsTable->createobject;
-          gNetscapeFuncs.retainobject = nsTable->retainobject;
-          gNetscapeFuncs.releaseobject = nsTable->releaseobject;
-          gNetscapeFuncs.invoke = nsTable->invoke;
-          gNetscapeFuncs.invokeDefault = nsTable->invokeDefault;
-          gNetscapeFuncs.evaluate = nsTable->evaluate;
-          gNetscapeFuncs.getproperty = nsTable->getproperty;
-          gNetscapeFuncs.setproperty = nsTable->setproperty;
-          gNetscapeFuncs.removeproperty = nsTable->removeproperty;
-          gNetscapeFuncs.hasproperty = nsTable->hasproperty;
-          gNetscapeFuncs.hasmethod = nsTable->hasmethod;
-          gNetscapeFuncs.releasevariantvalue = nsTable->releasevariantvalue;
-          gNetscapeFuncs.setexception = nsTable->setexception;
-        }
-
-        /* Version 0.16 */
-        if (nsTable->size >=
-            ((char *)&nsTable->poppopupsenabledstate - (char *)nsTable))
-        {
-          gNetscapeFuncs.pushpopupsenabledstate = nsTable->pushpopupsenabledstate;
-          gNetscapeFuncs.poppopupsenabledstate  = nsTable->poppopupsenabledstate;
-        }
-
-        /* Version 0.18 */
-        if (nsTable->size >=
-            ((char *)&nsTable->enumerate - (char *)nsTable))
-        {
-          gNetscapeFuncs.enumerate = nsTable->enumerate;
-        }
-
-        /* Version 0.19 */
-        if (nsTable->size >=
-            ((char *)&nsTable->pluginthreadasynccall - (char *)nsTable))
-        {
-          gNetscapeFuncs.pluginthreadasynccall = nsTable->pluginthreadasynccall;
-        }
-
-        /* Version 0.19?? */
-        if (nsTable->size >=
-            ((char *)&nsTable->construct - (char *)nsTable))
-        {
-          gNetscapeFuncs.construct = nsTable->construct;
-        }
-
-        /* Version 0.20 */
-        if (nsTable->size >=
-            ((char *)&nsTable->setvalueforurl - (char *)nsTable))
-        {
-          gNetscapeFuncs.getvalueforurl = nsTable->getvalueforurl;
-          gNetscapeFuncs.setvalueforurl = nsTable->setvalueforurl;
-        }
-
-        /* Version 0.21 */
-        if (nsTable->size >=
-            ((char *)&nsTable->getauthenticationinfo - (char *)nsTable))
-        {
-          gNetscapeFuncs.getauthenticationinfo = nsTable->getauthenticationinfo;
-        }
-
-
-        /* Version 0.23 */
-        if (nsTable->size >=
-            ((char *)&nsTable->convertpoint - (char *)nsTable))
-        {
-          gNetscapeFuncs.scheduletimer = nsTable->scheduletimer;
-          gNetscapeFuncs.unscheduletimer = nsTable->unscheduletimer;
-          gNetscapeFuncs.popupcontextmenu = nsTable->popupcontextmenu;
-          gNetscapeFuncs.convertpoint = nsTable->convertpoint;
-        }
-
-
-        /*
-         * Set up the plugin function table that Netscape will use to
-         * call us.  Netscape needs to know about our version and size
-         * and have a UniversalProcPointer for every function we
-         * implement.
-         */
-        pluginFuncs->version    = (NP_VERSION_MAJOR << 8) + NP_VERSION_MINOR;
-        pluginFuncs->newp          = NPP_New;
-        pluginFuncs->destroy       = NPP_Destroy;
-        pluginFuncs->setwindow     = NPP_SetWindow;
-        pluginFuncs->newstream     = NPP_NewStream;
-        pluginFuncs->destroystream = NPP_DestroyStream;
-        pluginFuncs->asfile        = NPP_StreamAsFile;
-        pluginFuncs->writeready    = NPP_WriteReady;
-        pluginFuncs->write         = NPP_Write;
-        pluginFuncs->print         = NPP_Print;
-        pluginFuncs->event         = NULL;
-        pluginFuncs->urlnotify     = NPP_URLNotify;
-
-        pluginFuncs->javaClass  = NULL; /* = NPP_GetJavaClass(); deprecated */
-
-        /* All this is so we keep backwards compatibility with old browsers! */ 
-        if (pluginFuncs->size > ((char*)&pluginFuncs->getvalue - (char*)pluginFuncs))
-        {     
-            pluginFuncs->getvalue = (NPP_GetValueProcPtr) NPP_GetValue;
-            if (pluginFuncs->size > ((char*)&pluginFuncs->setvalue - (char*)pluginFuncs))
-            {    
-                pluginFuncs->setvalue = NPP_SetValue;
-            }   
-        }
+        /* Zero to start */
+        memset(&funcs, 0, sizeof(NPPluginFuncs));
 
+        /* First create a local copy of the table */
+        funcs.version    = (NP_VERSION_MAJOR << 8) + NP_VERSION_MINOR;
+        funcs.newp          = NPP_New;
+        funcs.destroy       = NPP_Destroy;
+        funcs.setwindow     = NPP_SetWindow;
+        funcs.newstream     = NPP_NewStream;
+        funcs.destroystream = NPP_DestroyStream;
+        funcs.asfile        = NPP_StreamAsFile;
+        funcs.writeready    = NPP_WriteReady;
+        funcs.write         = NPP_Write;
+        funcs.print         = NPP_Print;
+        funcs.urlnotify     = NPP_URLNotify;
+        funcs.getvalue      = (NPP_GetValueProcPtr) NPP_GetValue;
+        funcs.setvalue      = NPP_SetValue;
+               
         if(pluginFuncs->size > sizeof(NPPluginFuncs))
         {   
-            /* Do this so browser knows the upper limit of plugin functions supported */
-            pluginFuncs->size = sizeof(NPPluginFuncs);
-        } 
+            /* Zero the fields we dont have anything for */
+            memset(&pluginFuncs[sizeof(NPPluginFuncs)], 0, 
+                                     pluginFuncs->size - sizeof(NPPluginFuncs));
+            funcs.size = sizeof(NPPluginFuncs);
+        }
+        else
+        {
+            funcs.size = pluginFuncs->size;
+        }
+
+        /* Copy across */
+        memcpy(pluginFuncs, &funcs, funcs.size);
+    }
+    else
+    {
+        err = NPERR_INVALID_FUNCTABLE_ERROR;
+    }
+
+    if(err == NPERR_NO_ERROR)
+    {
         err = NPP_Initialize();
     }
     
